/* 
   This file is part of TwinLib (http://rio.xdev.org/twinlib)
   Copyright (C) 1999-2000 Nicolas Roard (nroard@e-motive.com)
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef __PAINTLIB_H__
#define __PAINTLIB_H__

//#include <qapplication.h>
//#include <qwidget.h>
//#include <qpixmap.h>
//#include <qpicture.h>
#include <qcolor.h>
#include <qimage.h>
#include <qpainter.h>
//#include <qdrawutl.h>
#include <qpngio.h>
#include <qjpegio.h>
//#include <iostream.h>
//#include <stdio.h>

#define HGRADIENT2 0
#define VGRADIENT2 1
#define HGRADIENT3 3
#define VGRADIENT3 4

#define HGRAD 0
#define VGRAD 1

/*!
	\mainpage
	This library is intented to make graphic programming
	easier under Qt.
 
	It include graphics primitives like text, line, circle,
	and have an alphachannel support (gradients, antialias...),
	with Image, a class manipulation which encapsulate a QImage.
 
	\warning The <a href="http://www.troll.no">Qt lib</a> is required to use PaintLib.
	
	\author <a href="mailto:nroard@e-motive.com">Nicolas Roard</a>
	\date 17/01/99
*/

/*!
	\class Image
	\brief main class for PaintLib.
 
	You must declare an Image object to work with PaintLib.
	Basically, the class Image contain mImage, a QImage object, 
	and many members functions to work on it.
	
	example : 
	\include main.cpp
	<center>
	\image html example.jpg 
	\image latex example.eps width=15cm 
	</center>
	
*/
class Image
{

public :

    /*!
    	Constructor of the class

    	The constructor just create a new QImage object for mImage.
    */
    Image ()
    {
        mImage = QImage ();
    }

    /*!
    	\overload
    	\param img We initialize mImage with a QImage
    */
    Image (QImage img)
    {
        mImage = img;
    }

    /*!
    	\overload
    	Construct an image with width, height and depth parameters.
    	\param w width of the image
    	\param h height of the image
    	\param depth depth of the image (by default, 32bits)
    */
    Image (int w, int h, int depth = 32)
    {
        mImage = QImage (w, h, depth);
    }

    /*!
    	\overload 
    	Initialize mImage with an image file (jpeg or png for example)
    */
    Image (char* path)
    {
        mImage = QImage (path);
    }

    /*!
    	return mImage (QImage object) 
    */
    QImage data ()
    {
        return mImage;
    }

    /*!
    	do a simple blur on mImage
    	\param blur level of blur (number of pass of the blur filter)
    */
    void blur (int blur);

    /*!
    	draw a nice relief frame around the image
    	\param Relief type of "relief" for the frame
    	\param ReliefWidth width of the border
    	\param FactorR level of saturation/desaturation (64 for example)
    */
    void frame (int Relief, int ReliefWidth, int FactorR);

    /*!
    	draw a text, normal or antialiased 
    	\param text the string to draw
    	\param font a QFont to set the style of the drawing
    	\param x position in X of the text
    	\param y position in Y of the text 
    	\param colorTexte color of the text
    	\param typeAntialias 0 to draw a normal text and 1 to draw an antialiased text
    	\param fondImage set to TRUE if you draw a text over an image (and not an single-color filled image)

			\warning This function only works if an object qApp exist, ie, no utilisation possible in text mode (without X connection)
    */
    void text (QString text, QFont font, int x, int y, QColor colorTexte, int typeAntialias = 0, bool fondImage = TRUE);

    /*!
    	do a gradient in the alpha channel of the image
    	\param intensite_debut start level of the alpha gradient
    	\param intensite_milieu eventually, mid level of the alpha gradient
    	\param intensite_fin end level of the alpha gradient
    	\param type type of gradient : HGRADIENT1, HGRADIENT2, VGRADIENT2, VGRADIENT3.
    	\param width width of the gradient
    	\param height height of the gradient
    	
    	The type of gradient differs by the direction (H : Horizontal, V : Vertical), and the
    	number set the number of used intensities.
    */
    void alphaGradient (int intensite_debut, int intensite_milieu, int intensite_fin, int Type, int width, int height);

    /*!
    	do a gradient on the image
    	\param StartColor start color of the gradient
    	\param MidColor eventually, the mid color of the gradient
    	\param EndColor end color of the gradient
    	\param type type of gradient : HGRADIENT1, HGRADIENT2, VGRADIENT2, VGRADIENT3.
    	\param width width of the gradient
    	\param height height of the gradient
    	
    	The type of gradient differs by the direction (H : Horizontal, V : Vertical), and the
    	number set the number of used colors.
    */
    void gradient (QColor StartColor, QColor MidColor, QColor EndColor, int type, int width, int height);

    /*!
    	do a copy using alphachannel
    	\param imageSrc the source image to copy on mImage
    	\param x position X to copy the source on mImage
    	\param y position X to copy the source on mImage
    */
    void alphaCopy (QImage* imageSrc, int x = 0, int y = 0);

    /*!
    	\overload 
    	\param imageSrc the source image to copy on mImage
    	\param x position X to copy the source on mImage
    	\param y position X to copy the source on mImage
    */
    void alphaCopy (Image* imageSrc, int x = 0, int y = 0);

    /*!
    	do a simple copy (without alphachannel) 
    	\param imageSrc the source image to copy on mImage
    	\param x position X to copy the source on mImage
    	\param y position X to copy the source on mImage
    	\param mode if mode != 0 then this function only copy pixels others than white .
    */
    void copy (QImage* imageSrc, int x = 0, int y = 0, int mode = 0);

    /*!
    	\overload 
    	\param imageSrc the source image to copy on mImage
    	\param x position X to copy the source on mImage
    	\param y position X to copy the source on mImage
    	\param mode if mode != 0 then this function only copy pixels others than white .
    */
    void copy (Image* imageSrc, int x = 0, int y = 0, int mode = 0);

    /*!
    	draw an antialiased line
    	\param x1 X composant of first point of the line
    	\param y1 Y composant of first point of the line
    	\param x2 X composant of last point of the line
    	\param y2 Y composant of last point of the line
    	\param c drawing color
    */
    void aline (int x1, int y1, int x2, int y2, QColor c);

    /*!
    	draw an antialiased line with a certain width 
    	\param x1 X composant of first point of the line
    	\param y1 Y composant of first point of the line
    	\param x2 X composant of last point of the line
    	\param y2 Y composant of last point of the line
    	\param c drawing color
    	\param width width of the line
    */
    void aline (int x1, int y1, int x2, int y2, QColor c, int width);

    /*!
    	draw a classic line (without antialiasing)
    	\param x1 X composant of first point of the line
    	\param y1 Y composant of first point of the line
    	\param x2 X composant of last point of the line
    	\param y2 Y composant of last point of the line
    	\param c drawing color
    */
    void line (int x1, int y1, int x2, int y2, QColor c);

    /*!
     	draw a classic line with a certain width	
    	\param x1 X composant of first point of the line
    	\param y1 Y composant of first point of the line
    	\param x2 X composant of last point of the line
    	\param y2 Y composant of last point of the line
    	\param c drawing color
    	\param width width of the line
    */
    void line (int x1, int y1, int x2, int y2, QColor c, int width);

    /*!
    	draw a simple box 
    	\param x top left X position
    	\param y top left Y position
    	\param w width of the box
    	\param h height of the box
    	\param c drawing color
    */
    void box (int x, int y, int w, int h, QColor c);

    /*!
    	draw a plain box 
    	\param x top left X position
    	\param y top left Y position
    	\param w width of the box
    	\param h height of the box
    	\param c drawing color
    */
    void pbox (int x, int y, int w, int h, QColor c);

    /*!
    	draw a plain box with an alpha value 
    	\param x top left X position
    	\param y top left Y position
    	\param w width of the box
    	\param h height of the box
    	\param c drawing color
    	\param alpha degree of translucent
    */
    void pboxa (int x, int y, int w, int h, QColor c, float alpha);
		
    /*!
    	draw a plain box with 3-colors gradient 
    	\param x top left X position
    	\param y top left Y position
    	\param w width of the box
    	\param h height of the box
    	\param start start color of the gradient 
    	\param mid mid color of the gradient 
    	\param end end color of the gradient 
    	\param mode style of gradient : VGRAD or HGRAD 
    */
		void pboxg (int x, int y, int w, int h, QColor start, QColor mid, QColor end, int mode);

    /*!
    	draw a plain box with 3-colors gradient 
    	\param x top left X position
    	\param y top left Y position
    	\param w width of the box
    	\param h height of the box
    	\param start start color of the gradient 
    	\param mid mid color of the gradient 
    	\param end end color of the gradient 
    	\param mode style of gradient : VGRAD or HGRAD 
    	\param alpha degree of translucent
    */
		void pboxga (int x, int y, int w, int h, QColor start, QColor mid, QColor end, int mode, float alpha);


    /*!
    	draw a circle
    	\param centerX X composant of the center position of the circle
    	\param centerY Y composant of the center position of the circle
    	\param r radius of the circle
    	\param c drawing color
    */
    void circle (int centerX, int centerY, int r, QColor c);

    /*!
    	draw an antialiased circle
    	\param centerX X composant of the center position of the circle
    	\param centerY Y composant of the center position of the circle
    	\param r radius of the circle
    	\param c drawing color
    */
    void acircle (int centerX, int centerY, int r, QColor c);

    /*!
    	draw a plain circle 
    	\param centerX X composant of the center position of the circle
    	\param centerY Y composant of the center position of the circle
    	\param r radius of the circle
    	\param c drawing color
    */
    void pcircle (int centerX, int centerY, int r, QColor c);

    /*!
    	draw a plain circle with an alpha value 
    	\param centerX X composant of the center position of the circle
    	\param centerY Y composant of the center position of the circle
    	\param r radius of the circle
    	\param c drawing color
    	\param alpha degree of translucent
    */
    void pcirclea (int centerX, int centerY, int r, QColor c, float alpha);

    /*!
    	draw a simple ellipse.
    	\param xc X composant of the center position of the ellipse
    	\param yc Y composant of the center position of the ellipse
    	\param a  width composant of the ellipse
    	\param b  height composant of the ellipse
    	\param c drawing color
    */
    void ellipse (int xc, int yc, int a, int b, QColor c);

    /*!
    	draw an antialiased ellipse.
    	\param xc X composant of the center position of the ellipse
    	\param yc Y composant of the center position of the ellipse
    	\param a  width composant of the ellipse
    	\param b  height composant of the ellipse
    	\param c drawing color
    */
    void aellipse (int xc, int yc, int a, int b, QColor c);


    /*!
    	an useful function for working with alphachannel
    	\param r red composant
    	\param g green composant
    	\param b blue composant
    	\param a alpha composant (degree of translucent)
    */
    QRgb Rgba( int r, int g, int b, int a);

    /*! return the height of the data (QImage) */
    int height()
    {
        return mImage.height();
    }

    /*! return the width of the data (QImage) */
    int width()
    {
        return mImage.width();
    }

private :

    /*! private data, the QImage we are working on */
    QImage mImage;
    void traceAEllipse (int xinc, int yinc, int xc, int yc, int a, int b, QColor c);
    void traceEllipse (int xinc, int yinc, int xc, int yc, int a, int b, QColor c);
};

#endif   // __PAINTLIB_H__
